@@ -313,6 +313,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             log("border color derived from %s: %s", extra_args, color_str)
         try:
             color = color_parse(color_str)
+            assert color is not None
         except Exception as e:
             log.warn("invalid border color specified: '%s' (%s)", color_str, e)
             border_help()
@@ -1104,6 +1105,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             #GTK2 on OSX doesn't do alpha:
             if metadata.boolget("has-alpha", False):
                 return (self.ClientWindowClass,)
+            if not metadata.boolget("decorations", True):
+                return (self.ClientWindowClass,)
         return (self.GLClientWindowClass, self.ClientWindowClass)
 
     def toggle_opengl(self, *_args):
