@@ -12,7 +12,7 @@ try:
 except ImportError:
     from urllib.parse import unquote    #python3 @Reimport @UnresolvedImport
 
-from xpra.os_util import bytestostr, is_X11, monotonic_time, WIN32, OSX, POSIX, PYTHON3
+from xpra.os_util import bytestostr, strtobytes, is_X11, monotonic_time, WIN32, OSX, POSIX, PYTHON3, PYTHON2
 from xpra.util import (
     AdHocStruct, typedict, envint, envbool, nonl, csv, first_time,
     WORKSPACE_UNSET, WORKSPACE_ALL, WORKSPACE_NAMES, MOVERESIZE_DIRECTION_STRING, SOURCE_INDICATION_STRING,
@@ -929,15 +929,17 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
     def set_x11_property(self, prop_name, dtype, dformat, value):
         metalog("set_x11_property%s", (prop_name, dtype, dformat, value))
-        gdk_window = self.get_window()
-        if not dtype and not dformat:
-            #remove prop
-            prop_del(gdk_window, prop_name)
-            return
         dtype = bytestostr(dtype)
         if dtype=="latin1":
             value = bytestostr(value)
-        prop_set(gdk_window, prop_name, dtype, value)
+        def do_set_prop():
+            gdk_window = self.get_window()
+            if not dtype and not dformat:
+                #remove prop
+                prop_del(gdk_window, prop_name)
+            else:
+                prop_set(gdk_window, prop_name, dtype, value)
+        self.when_realized("x11-prop-%s" % prop_name, do_set_prop)
 
     def set_class_instance(self, wmclass_name, wmclass_class):
         if not self.is_realized():
@@ -948,7 +950,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         elif HAS_X11_BINDINGS:
             xid = get_xwindow(self.get_window())
             with xlog:
-                X11Window.setClassHint(xid, wmclass_class, wmclass_name)
+                X11Window.setClassHint(xid, strtobytes(wmclass_class), strtobytes(wmclass_name))
                 log("XSetClassHint(%s, %s) done", wmclass_class, wmclass_name)
 
     def set_shape(self, shape):
@@ -1915,8 +1917,8 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
             y += self.window_offset[1]
             #TODO: check this doesn't move it off-screen!
         self._resize_counter = resize_counter
-        window = self.get_window()
-        if window.get_position()==(x, y):
+        wx, wy = self.get_drawing_area_geometry()[:2]
+        if (wx, wy)==(x, y):
             #same location, just resize:
             if self._size==(w, h):
                 geomlog("window unchanged")
@@ -1930,6 +1932,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
             geomlog("window was not realized yet")
             self.realize()
         #adjust for window frame:
+        window = self.get_window()
         ox, oy = window.get_origin()[-2:]
         rx, ry = window.get_root_origin()
         ax = x - (ox - rx)
