@@ -1,18 +1,18 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 
-from xpra.log import Logger
-log = Logger("sound")
-
 from xpra.net.compression import Compressed
 from xpra.server.source.stub_source_mixin import StubSourceMixin
 from xpra.os_util import get_machine_id, get_user_uuid, bytestostr, POSIX
-from xpra.util import csv, envbool, flatten_dict, XPRA_AUDIO_NOTIFICATION_ID
+from xpra.util import csv, envbool, flatten_dict, typedict, XPRA_AUDIO_NOTIFICATION_ID
+from xpra.log import Logger
+
+log = Logger("sound")
 
 NEW_STREAM_SOUND = envbool("XPRA_NEW_STREAM_SOUND", True)
 
@@ -69,7 +69,8 @@ class AudioMixin(StubSourceMixin):
         self.sound_send = c.boolget("sound.send")
         self.sound_bundle_metadata = c.boolget("sound.bundle-metadata")
         log("pulseaudio id=%s, cookie-hash=%s, server=%s, sound decoders=%s, sound encoders=%s, receive=%s, send=%s",
-                 self.pulseaudio_id, self.pulseaudio_cookie_hash, self.pulseaudio_server, self.sound_decoders, self.sound_encoders, self.sound_receive, self.sound_send)
+                 self.pulseaudio_id, self.pulseaudio_cookie_hash, self.pulseaudio_server,
+                 self.sound_decoders, self.sound_encoders, self.sound_receive, self.sound_send)
 
     def get_caps(self):
         if not self.wants_sound or not self.sound_properties:
@@ -83,7 +84,7 @@ class AudioMixin(StubSourceMixin):
             "receive"           : self.supports_microphone and len(self.microphone_codecs)>0,
             })
         return flatten_dict({"sound" : sound_props})
-        
+
 
     def audio_loop_check(self, mode="speaker"):
         log("audio_loop_check(%s)", mode)
@@ -92,19 +93,26 @@ class AudioMixin(StubSourceMixin):
             return True
         machine_id = get_machine_id()
         uuid = get_user_uuid()
-        log("audio_loop_check(%s) machine_id=%s client machine_id=%s, uuid=%s, client uuid=%s", mode, machine_id, self.machine_id, uuid, self.uuid)
-        if self.machine_id:
-            if self.machine_id!=machine_id:
+        #these attributes belong in a different mixin,
+        #so we can't assume that they exist:
+        client_machine_id = getattr(self, "machine_id", None)
+        client_uuid = getattr(self, "uuid", None)
+        log("audio_loop_check(%s) machine_id=%s client machine_id=%s, uuid=%s, client uuid=%s",
+            mode, machine_id, client_machine_id, uuid, client_uuid)
+        if client_machine_id:
+            if client_machine_id!=machine_id:
                 #not the same machine, so OK
                 return True
-            if self.uuid!=uuid:
+            if client_uuid!=uuid:
                 #different user, assume different pulseaudio server
                 return True
         #check pulseaudio id if we have it
         pulseaudio_id = self.sound_properties.get("pulseaudio", {}).get("id")
         pulseaudio_cookie_hash = self.sound_properties.get("pulseaudio", {}).get("cookie-hash")
-        log("audio_loop_check(%s) pulseaudio id=%s, client pulseaudio id=%s, pulseaudio cookie hash=%s, client pulseaudio cookie hash=%s",
-                 mode, pulseaudio_id, self.pulseaudio_id, pulseaudio_cookie_hash, self.pulseaudio_cookie_hash)
+        log("audio_loop_check(%s) pulseaudio id=%s, client pulseaudio id=%s",
+                 mode, pulseaudio_id, self.pulseaudio_id)
+        log("audio_loop_check(%s) pulseaudio cookie hash=%s, client pulseaudio cookie hash=%s",
+                 mode, pulseaudio_cookie_hash, self.pulseaudio_cookie_hash)
         if pulseaudio_id and self.pulseaudio_id:
             if self.pulseaudio_id!=pulseaudio_id:
                 return True
@@ -125,41 +133,43 @@ class AudioMixin(StubSourceMixin):
             log.warn(" %s", x)
         return False
 
-    def start_sending_sound(self, codec=None, volume=1.0, new_stream=None, new_buffer=None, skip_client_codec_check=False):
-        assert self.hello_sent
+    def start_sending_sound(self, codec=None, volume=1.0,
+                            new_stream=None, new_buffer=None, skip_client_codec_check=False):
         log("start_sending_sound(%s)", codec)
         ss = None
+        if getattr(self, "suspended", False):
+            log.warn("Warning: not starting sound whilst in suspended state")
+            return None
+        if not self.supports_speaker:
+            log.error("Error sending sound: support not enabled on the server")
+            return None
+        if self.sound_source:
+            log.error("Error sending sound: forwarding already in progress")
+            return None
+        if not self.sound_receive:
+            log.error("Error sending sound: support is not enabled on the client")
+            return None
+        if codec is None:
+            codecs = [x for x in self.sound_decoders if x in self.speaker_codecs]
+            if not codecs:
+                log.error("Error sending sound: no codecs in common")
+                return None
+            codec = codecs[0]
+        elif codec not in self.speaker_codecs:
+            log.warn("Warning: invalid codec specified: %s", codec)
+            return None
+        elif (codec not in self.sound_decoders) and not skip_client_codec_check:
+            log.warn("Error sending sound: invalid codec '%s'", codec)
+            log.warn(" is not in the list of decoders supported by the client: %s", csv(self.sound_decoders))
+            return None
+        if not self.audio_loop_check("speaker"):
+            return None
         try:
-            if getattr(self, "suspended", False):
-                log.warn("Warning: not starting sound whilst in suspended state")
-                return None
-            if not self.supports_speaker:
-                log.error("Error sending sound: support not enabled on the server")
-                return None
-            if self.sound_source:
-                log.error("Error sending sound: forwarding already in progress")
-                return None
-            if not self.sound_receive:
-                log.error("Error sending sound: support is not enabled on the client")
-                return None
-            if codec is None:
-                codecs = [x for x in self.sound_decoders if x in self.speaker_codecs]
-                if not codecs:
-                    log.error("Error sending sound: no codecs in common")
-                    return None
-                codec = codecs[0]
-            elif codec not in self.speaker_codecs:
-                log.warn("Warning: invalid codec specified: %s", codec)
-                return None
-            elif (codec not in self.sound_decoders) and not skip_client_codec_check:
-                log.warn("Error sending sound: invalid codec '%s'", codec)
-                log.warn(" is not in the list of decoders supported by the client: %s", csv(self.sound_decoders))
-                return None
-            if not self.audio_loop_check("speaker"):
-                return None
             from xpra.sound.wrapper import start_sending_sound
             plugins = self.sound_properties.strlistget("plugins", [])
-            ss = start_sending_sound(plugins, self.sound_source_plugin, None, codec, volume, True, [codec], self.pulseaudio_server, self.pulseaudio_id)
+            ss = start_sending_sound(plugins, self.sound_source_plugin,
+                                     None, codec, volume, True, [codec],
+                                     self.pulseaudio_server, self.pulseaudio_id)
             self.sound_source = ss
             log("start_sending_sound() sound source=%s", ss)
             if not ss:
@@ -201,11 +211,13 @@ class AudioMixin(StubSourceMixin):
         if ss:
             self.sound_source = None
             self.send_eos(ss.codec, ss.sequence)
+            self.sound_source_sequence += 1
             ss.cleanup()
 
     def send_eos(self, codec, sequence=0):
+        log("send_eos(%s, %s)", codec, sequence)
         #tell the client this is the end:
-        self.send_more("sound-data", codec, "", 
+        self.send_more("sound-data", codec, "",
                        {
                            "end-of-stream" : True,
                            "sequence"      : sequence,
@@ -223,13 +235,18 @@ class AudioMixin(StubSourceMixin):
                         sink = "alsasink"
                     else:
                         sink = "autoaudiosink"
-                    cmd = ["gst-launch-1.0", "-q", "filesrc", "location=%s" % sample, "!", "decodebin", "!", "audioconvert", "!", sink]
+                    cmd = [
+                        "gst-launch-1.0", "-q",
+                        "filesrc", "location=%s" % sample,
+                        "!", "decodebin",
+                        "!", "audioconvert",
+                        "!", sink]
                     import subprocess
                     proc = subprocess.Popen(cmd, close_fds=True)
                     log("Popen(%s)=%s", cmd, proc)
                     from xpra.child_reaper import getChildReaper
                     getChildReaper().add_process(proc, "new-stream-sound", cmd, ignore=True, forget=True)
-            except:
+            except Exception:
                 pass
         log("new_stream(%s, %s)", sound_source, codec)
         if self.sound_source!=sound_source:
@@ -244,22 +261,29 @@ class AudioMixin(StubSourceMixin):
                    "codec"              : codec,
                    "sequence"           : sound_source.sequence,
                    })
-        #self.update_av_sync_delay_total()
+        update_av_sync = getattr(self, "update_av_sync_delay_total", None)
+        if update_av_sync:
+            update_av_sync()  #pylint: disable=not-callable
+            #run it again after 10 seconds,
+            #by that point the source info will actually be populated:
+            from gi.repository import GLib
+            GLib.timeout_add(10*1000, update_av_sync)
 
-    def new_sound_buffer(self, sound_source, data, metadata, packet_metadata=[]):
+    def new_sound_buffer(self, sound_source, data, metadata, packet_metadata=None):
         log("new_sound_buffer(%s, %s, %s, %s) info=%s",
                  sound_source, len(data or []), metadata, [len(x) for x in packet_metadata], sound_source.info)
         if self.sound_source!=sound_source or self.is_closed():
             log("sound buffer dropped: from old source or closed")
             return
         if sound_source.sequence<self.sound_source_sequence:
-            log("sound buffer dropped: old sequence number: %s (current is %s)", sound_source.sequence, self.sound_source_sequence)
+            log("sound buffer dropped: old sequence number: %s (current is %s)",
+                sound_source.sequence, self.sound_source_sequence)
             return
         if packet_metadata:
             if not self.sound_bundle_metadata:
                 #client does not support bundling, send packet metadata as individual packets before the main packet:
                 for x in packet_metadata:
-                    self.send_sound_data(sound_source, x)
+                    self.send_sound_data(sound_source, x, {})
                 packet_metadata = ()
             else:
                 #the packet metadata is compressed already:
@@ -268,7 +292,7 @@ class AudioMixin(StubSourceMixin):
         can_drop_packet = (sound_source.info or {}).get("buffer_count", 0)>10
         self.send_sound_data(sound_source, data, metadata, packet_metadata, can_drop_packet)
 
-    def send_sound_data(self, sound_source, data, metadata={}, packet_metadata=(), can_drop_packet=False):
+    def send_sound_data(self, sound_source, data, metadata, packet_metadata=None, can_drop_packet=False):
         packet_data = [sound_source.codec, Compressed(sound_source.codec, data), metadata]
         if packet_metadata:
             assert self.sound_bundle_metadata
@@ -296,7 +320,6 @@ class AudioMixin(StubSourceMixin):
     ##########################################################################
     # sound control commands:
     def sound_control(self, action, *args):
-        assert self.hello_sent
         action = bytestostr(action)
         log("sound_control(%s, %s)", action, args)
         method = getattr(self, "sound_control_%s" % (action.replace("-", "_")), None)
@@ -304,7 +327,7 @@ class AudioMixin(StubSourceMixin):
             msg = "unknown sound action: %s" % action
             log.error(msg)
             return msg
-        return method(*args)
+        return method(*args)  #pylint: disable=not-callable
 
     def sound_control_stop(self, sequence_str=""):
         if sequence_str:
@@ -346,6 +369,7 @@ class AudioMixin(StubSourceMixin):
 
     def do_sound_control_start(self, volume, codec):
         codec = bytestostr(codec)
+        log("do_sound_control_start(%s, %s)", volume, codec)
         if not self.start_sending_sound(codec, volume):
             return "failed to start sound"
         msg = "sound started"
@@ -386,7 +410,8 @@ class AudioMixin(StubSourceMixin):
             self.source_remove(sft)
 
     def sound_data(self, codec, data, metadata, packet_metadata=()):
-        log("sound_data(%s, %s, %s, %s) sound sink=%s", codec, len(data or []), metadata, packet_metadata, self.sound_sink)
+        log("sound_data(%s, %s, %s, %s) sound sink=%s",
+            codec, len(data or []), metadata, packet_metadata, self.sound_sink)
         if self.is_closed():
             return
         if self.sound_sink is not None and codec!=self.sound_sink.codec:
@@ -403,7 +428,7 @@ class AudioMixin(StubSourceMixin):
                 from xpra.util import AdHocStruct
                 self.sound_sink = AdHocStruct()
                 self.sound_sink.codec = codec
-                def noop(*args):
+                def noop(*_args):
                     pass
                 self.sound_sink.add_data = noop
                 self.sound_sink.cleanup = noop
@@ -438,11 +463,14 @@ class AudioMixin(StubSourceMixin):
         ss = self.sound_source
         cinfo = ""
         if ss:
+            info = typedict(ss.info or {})
             try:
-                encoder_latency = ss.info.get("queue", {}).get("cur", 0)
-                log("server side queue level: %s", encoder_latency)
+                qdict = info.dictget("queue")
+                if qdict:
+                    q = typedict(qdict).intget("cur", 0)
+                    log("server side queue level: %s", q)
                 #get the latency from the source info, if it has it:
-                encoder_latency = ss.info.get("latency", -1)
+                encoder_latency = info.intget("latency", -1)
                 if encoder_latency<0:
                     #fallback to hard-coded values:
                     from xpra.sound.gstreamer_util import ENCODER_LATENCY, RECORD_PIPELINE_LATENCY
